/// @addtogroup homologicalAlgebra
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file FGAGrpSignature.h
///
/// @author Marian Mrozek
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2006 by the CAPD Group.
//
// This file constitutes a part of the CAPD library, 
// distributed under the terms of the GNU General Public License.
// Consult  http://capd.wsb-nlu.edu.pl/ for details. 

#if !defined(_FGAGRPSIGNATURE_H_)
#define _FGAGRPSIGNATURE_H_

#include <vector>
#include <sstream>
#include <string>

class FGAGrpSignature{
  int m_rank;
  std::vector<int> m_torsionCoef;
public:

  FGAGrpSignature(int A_rank=0):m_rank(A_rank){}
  FGAGrpSignature(int A_rank, std::vector<int>& A_torsionCoef):m_rank(A_rank){
    m_torsionCoef=A_torsionCoef;
  }

  int rank() const{return m_rank;}
  int numberOfTorsionCoefs() const{return m_torsionCoef.size();}
  int torsionCoef(int i) const{ return m_torsionCoef[i];}
  bool isTrivial(){
    return rank() == 0 and numberOfTorsionCoefs() == 0;
  }
  FGAGrpSignature& operator+=(const FGAGrpSignature& fga2){
    m_rank+=fga2.m_rank;
    m_torsionCoef.insert(m_torsionCoef.end(),fga2.m_torsionCoef.begin(),fga2.m_torsionCoef.end());
    return *this;
  }
  friend std::ostream& operator<<(std::ostream& out, const FGAGrpSignature& fga){
    if(fga.m_torsionCoef.size()==0){
      if(fga.m_rank) out << "Z^" << fga.m_rank;
      else out << "0";
    }else{
      if(fga.m_rank) out << "Z^" << fga.m_rank << " + ";
      for(int i=0;i<(int)fga.m_torsionCoef.size();++i){
        out << "Z/" << fga.m_torsionCoef[i];
        if(i<(int)fga.m_torsionCoef.size()-1) out << " + ";
      }
    }
    return out;
  }
};

#endif //_FGAGRPSIGNATURE_H_
/// @}

